package com.commtrain.PMS;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.net.URL;
import java.net.URLDecoder;
import java.util.Enumeration;
import java.util.MissingResourceException;
import java.util.PropertyResourceBundle;
import java.util.ResourceBundle;

import org.apache.log4j.PropertyConfigurator;

/**
 * 
 */
public class Props {

    private static org.apache.log4j.Logger logger = org.apache.log4j.Logger.getLogger(Props.class);

    /**
     * File name for the resource bundle
     */
    private static String fileName;

    /**
     * File path for the resource bundle
     */
    private static String filePath;

    /**
     * resource bundle
     */
    private static ResourceBundle resourceBundle;

    /**
     * This exception is thrown when something is missing
     */
    public static class NotFoundException extends Exception {

        /**
         * Default constructor
         */
        public NotFoundException() {
            super();
        }

        /**
         * Constructs a new NotFoundException with the specified error message
         * @param message the specified error message
         */
        public NotFoundException(String message) {
            super(message);
        }
    }

    /**
     * Default constructor
     */
    private Props() {
    }

    /**
     * @return fileName
     */
    public static String getFileName() {
        return fileName;
    }

    /**
     * @return filePath
     */
    public static String getFilePath() {
        return filePath;
    }

    /**
     * Sets resource bundle file name
     * @param fileName resource bundle file name
     * @throws NotFoundException
     *          If the resource bundle is not found
     * @throws IOException
     *          If an I/O error occurs
     */
    public static synchronized void setFileName(String fileName) throws IOException, NotFoundException {
        Props.fileName = fileName;
        ClassLoader loader = Props.class.getClassLoader();
        if (fileName == null) {
            throw new NotFoundException("Resource file not specified!");
        }
        URL url = loader.getResource(fileName);
        if (url != null) {
            filePath = URLDecoder.decode(url.getPath());
            InputStream inp = new FileInputStream(new File(filePath));
//          InputStream inp = url.openStream();
            resourceBundle = new PropertyResourceBundle(inp);
        } else {
            throw new NotFoundException("Resource file: '" + (fileName != null ? fileName : "") + "' not found!");
        }
    }

    /**
     * @throws NotFoundException
     *          If the resource bundle is not found
     * @return resourceBundle
     * 
     */
    protected static synchronized ResourceBundle getResourceBundle() throws NotFoundException {
        if (resourceBundle == null) {
            throw new NotFoundException("Not initialized.");
        }
        return resourceBundle;
    }

    /**
     * Returns property with specified key.
     * @param propertyName key whose associated value is to be returned
     * @return associated value
     * @throws NotFoundException if the resource bundle is not initialized
     * @throws MissingResourceException if the resource bundle does not contain the specified key
     */
    public static String get(String propertyName) throws NotFoundException, MissingResourceException {
        String value = getResourceBundle().getString(propertyName);
        return value;
    }

    /**
     * Returns property with specified key or the default value 
     * if the resource bundle does not contain the specified key.
     * @param propertyName key whose associated value is to be returned
     * @param defaultValue the default value
     * @return associated value
     */
    public static String get(String propertyName, String defaultValue) {
        try {
            String value = getResourceBundle().getString(propertyName);
            return value;
        } catch (Exception e) {
            return defaultValue;
        }
    }

    /**
     * Returns property with specified key.
     * @param propertyName key whose associated value is to be returned
     * @return associated value
     * @throws NotFoundException if the resource bundle is not initialized
     * @throws NumberFormatException if the property value can not be converted to an integer
     * @throws MissingResourceException if the resource bundle does not contain the specified key
     */
    public static int getInt(String propertyName)
        throws NotFoundException, NumberFormatException, MissingResourceException {
        String value = getResourceBundle().getString(propertyName);
        int intValue = Integer.parseInt(value);
        return intValue;
    }

    /**
     * Returns property with specified key or the default value 
     * if the resource bundle does not contain the specified key.
     * @param propertyName key whose associated value is to be returned
     * @param defaultValue the default value
     * @return associated value
     */
    public static int getInt(String propertyName, int defaultValue) {
        try {
            String value = getResourceBundle().getString(propertyName);
            int intValue = Integer.parseInt(value);
            return intValue;
        } catch (Exception e) {
            return defaultValue;
        }
    }

    /**
     * Returns property with specified key.
     * @param propertyName key whose associated value is to be returned
     * @return associated value
     * @throws NotFoundException if the resource bundle is not initialized
     * @throws NumberFormatException if the property value can not be converted to an integer
     * @throws MissingResourceException if the resource bundle does not contain the specified key
     */
    public static long getLong(String propertyName) throws
            NotFoundException, NumberFormatException, MissingResourceException {
        String value = getResourceBundle().getString(propertyName);
        long longValue = Long.parseLong(value);
        return longValue;
    }

    /**
     * Returns property with specified key or the default value 
     * if the resource bundle does not contain the specified key.
     * @param propertyName key whose associated value is to be returned
     * @param defaultValue the default value
     * @return associated value
     */
    public static long getLong(String propertyName, long defaultValue) {
        try {
            String value = getResourceBundle().getString(propertyName);
            long longValue = Long.parseLong(value);
            return longValue;
        } catch (Exception e) {
            return defaultValue;
        }
    }

    /**
     * Returns property with specified key.
     * @param propertyName key whose associated value is to be returned
     * @return associated value
     * @throws NotFoundException if the resource bundle is not initialized
     *          or the value can not be converted to boolean 
     * @throws MissingResourceException if the resource bundle does not contain the specified key
     */
    public static boolean getBoolean(String propertyName) throws NotFoundException, MissingResourceException {
        String value = getResourceBundle().getString(propertyName).trim();
        boolean trueValue = "true".equalsIgnoreCase(value) || "yes".equalsIgnoreCase(value) || "on".equalsIgnoreCase(value);
        boolean falseValue = "false".equalsIgnoreCase(value) || "no".equalsIgnoreCase(value) || "off".equalsIgnoreCase(value);
        if (trueValue | falseValue) {
            return trueValue;
        } else {
            throw new NotFoundException("Invalid boolean value: " + value);
        }
    }

    /**
     * Returns property with specified key or the default value 
     * if the resource bundle does not contain the specified key.
     * @param propertyName key whose associated value is to be returned
     * @param defaultValue the default value
     * @return associated value
     */
    public static boolean getBoolean(String propertyName, boolean defaultValue) {
        try {
            String value = Props.getResourceBundle().getString(propertyName).trim();
            boolean trueValue = "true".equalsIgnoreCase(value) || "yes".equalsIgnoreCase(value) || "on".equalsIgnoreCase(value);
            boolean falseValue = "false".equalsIgnoreCase(value) || "no".equalsIgnoreCase(value) || "off".equalsIgnoreCase(value);
            if (trueValue || falseValue) {
                return trueValue;
            } else {
                return defaultValue;
            }
        } catch (Exception e) {
            return defaultValue;
        }
    }

    /**
     * @see org.jpos.util.Loggeable#dump(PrintStream, String)
     */
    public static String dump() {
        String inner = "    ";
        String result = "\n<props>"
                        + "\n" + inner + "<filename>" + fileName + "</filename>";
        String key;
        String value;
        try {
            for (Enumeration props = getResourceBundle().getKeys(); props.hasMoreElements();) {
                key = (String) props.nextElement();
                value = resourceBundle.getString(key);
                result +=   "\n" + inner + "<item>"
                          + "\n" + inner + inner + "<name>" + key + "</name>"
                          + "\n" + inner + inner + "<value>" + value + "</value>"
                          + "\n" + inner + "</item>";
            }
            result += "\n</props>";
        } catch (NotFoundException e) {
            result = e.toString();
        }
        return result;
    }

    /**
     * Tests
     * @param args Command-line parameters
     * @throws NotFoundException
     *          If the resource bundle is not found
     * @throws IOException
     *          If an I/O error occurs
     */
    public static void main(String[] args) throws NotFoundException, IOException {
        Props.setFileName("localServer.properties");
        PropertyConfigurator.configure(Props.getFilePath());
        boolean useUI = Props.getBoolean("useUI");
        logger.info(new Boolean(useUI));
        boolean defaultBoolean = Props.getBoolean("dummy", false);
        logger.info(new Boolean(defaultBoolean));
//        boolean booleanDummy = getBoolean("dummy");
        int ienPort = Props.getInt("IENPort");
        logger.info(new Integer(ienPort));
        int defaultInt = Props.getInt("dummy", 142857);
        logger.info(new Integer(defaultInt));
        String ienHost = Props.get("IENHost");
        logger.info(ienHost);
        String defaultString = Props.get("dummy", "dummy");
        logger.info(defaultString);
        logger.info(Props.dump());
    }
}
