package com.commtrain.PMS.db;
import java.sql.*;
import java.text.SimpleDateFormat;
import java.math.*;
import org.jpos.iso.*;
import com.commtrain.PMS.transaction.*;

/**
 * This class is used to register terminal transactions.
 * It calls stored procedure REGISTER_REQUEST.
 * 
 */
public class TransactionRegister {

    private static org.apache.log4j.Logger logger = org.apache.log4j.Logger.getLogger(TransactionRegister.class);

    public static String AUTHORIZATION_NO_DEFAULT = "000000";

    /**
     * Return status from DB procedure.
     */
    private int status;

    /**
     * The primary key (in CDS table) of the registration record.
     */
    private BigDecimal cdsID;

    /**
     * Acquirer NII.
     */
    private int acqNII;

    /**
     * Acuirer manufacturer ID.
     */
    private int acqManufacturerID;

    /**
     * Terminal acquirer ID.
     */
    private String trmACQID;

    /**
     * CAR sequence number.
     */
    private int carSeqNo;

    /**
     * VU number.
     */
    private String vUNumber;
    
    /**
     * Card type.
     */
    private int cardType;

    /**
     * Terminal ID.
     */
    private String trmID;

    /**
     * Retrieval Reference Number.
     */
    private String retrievalRefNo;


    /**
     * Constructs TransactionRegister from ISO message.
     * 
     * @param isoRequest ISO request
     * @param carRecordType CAR record type
     * @param transactionDateTime transaction time stamp
     * @param statusSuccess CDS status when the request is registered
     * @param statusFailure return status if the regitration is not successful
     * 
     * @exception SQLException when a SQL error occurs
     * @exception ISOException when ISO request is incorrect
     */
    public TransactionRegister(ISOMsg isoRequest,
                                int carRecordType,
                                java.util.Date transactionDateTime,
                                int statusSuccess,
                                int statusFailure) throws SQLException, ISOException {
        this(
            null,
            TransactionUtils.getString(isoRequest, Transaction.TERMINAL_IDENTIFICATION, ""),
            TransactionUtils.getString(isoRequest, Transaction.INVOICE_ECR_REFNO, ""), 
            TransactionUtils.getString(isoRequest, Transaction.SYSTEM_TRACE_AUDIT_NUM, ""), 
            new BigDecimal(isoRequest.getString(Transaction.AMOUNT_TRANSACTION)),
            transactionDateTime,
            TransactionUtils.getDataCaptureIndicator(isoRequest), 
            TransactionUtils.getCardDetails(isoRequest),
            TransactionUtils.getTransactionCode(isoRequest),
            TransactionUtils.getCARTerminalType(isoRequest),
            carRecordType,
            AUTHORIZATION_NO_DEFAULT,
            TransactionUtils.getIntCurrencyCode(isoRequest),
            false,
            null, 
            statusSuccess,
            statusFailure);
    }

    /**
     * Constructs TransactionRegister from MOTO request.
     * 
     * @param terminalID terminal ID
     * @param invoiceNo invoice number
     * @param sysTraceAuditNo sysytem trace audit number
     * @param amount transaction amount
     * @param transactionDateTime transaction time stamp
     * @param dataCaptureIndicator data capure inducator
     * @param track2Data track 2 data
     * @param transactionCode transaction code
     * @param carTerminalType CAR terminal type
     * @param carRecordType CAR record type
     * @param approvalCodeIso approval code
     * @param submissionCurrency submission currency
     * @param isMoto specifies MOTO transaction
     * @param statusSuccess CDS status when the request is registered
     * @param statusFailure return status if the regitration is not successful
     * @throws SQLException when SQL error occurs
     */
    public TransactionRegister(
          String terminalID,
          String invoiceNo, 
          String sysTraceAuditNo, 
          BigDecimal amount,
          java.util.Date transactionDateTime,
          String dataCaptureIndicator, 
          String track2Data,      
          int transactionCode, 
          int carTerminalType, 
          int carRecordType,
          String approvalCodeIso,
          int submissionCurrency,
          boolean isMoto,
          int statusSuccess,
          int statusFailure) throws SQLException {
          this(
              null,
              terminalID,
              invoiceNo, 
              sysTraceAuditNo, 
              amount,
              transactionDateTime,
              dataCaptureIndicator, 
              track2Data,         
              transactionCode,
              carTerminalType,
              carRecordType,
              approvalCodeIso,
              submissionCurrency,
              isMoto,
              null,
              statusSuccess,
              statusFailure
           );
            
      }

    /**
     * Constructs TransactionRegister.
     * 
     * @param terminalID terminal ID
     * @param invoiceNo invoice number
     * @param sysTraceAuditNo sysytem trace audit number
     * @param amount transaction amount
     * @param transactionDateTime transaction time stamp
     * @param dataCaptureIndicator data capure inducator
     * @param track2Data track 2 data
     * @param transactionCode transaction code
     * @param carTerminalType CAR terminal type
     * @param carRecordType CAR record type
     * @param approvalCodeIso approval code
     * @param submissionCurrency submission currency
     * @param statusSuccess CDS status when the request is registered
     * @param statusFailure return status if the regitration is not successful
     * @throws SQLException when SQL error occurs
     */
    public TransactionRegister(
        String terminalID,
        String invoiceNo, 
        String sysTraceAuditNo, 
        BigDecimal amount,
        java.util.Date transactionDateTime,
        String dataCaptureIndicator, 
        String track2Data,      
        int transactionCode, 
        int carTerminalType, 
        int carRecordType,
        String approvalCodeIso,
        int submissionCurrency,
        int statusSuccess,
        int statusFailure) throws SQLException {
        this(
            null,
            terminalID,
            invoiceNo, 
            sysTraceAuditNo, 
            amount,
            transactionDateTime,
            dataCaptureIndicator, 
            track2Data,         
            transactionCode,
            carTerminalType,
            carRecordType,
            approvalCodeIso,
            submissionCurrency,
            false,
            null,
            statusSuccess,
            statusFailure
        );
    }


    /**
     * Constructs TransactionRegister.
     * Uses specified DB connection.
     * 
     * @param conn DB connection
     * @param terminalID terminal ID
     * @param invoiceNo invoice number
     * @param sysTraceAuditNo sysytem trace audit number
     * @param amount transaction amount
     * @param transactionDateTime transaction time stamp
     * @param dataCaptureIndicator data capure inducator
     * @param track2Data track 2 data
     * @param transactionCode transaction code
     * @param carTerminalType CAR terminal type
     * @param carRecordType CAR record type
     * @param approvalCodeIso approval code
     * @param submissionCurrency submission currency
     * @param statusSuccess CDS status when the request is registered
     * @param statusFailure return status if the regitration is not successful
     * @throws SQLException when SQL error occurs
     */
    public TransactionRegister(
        Connection conn,
        String terminalID,
        String invoiceNo, 
        String sysTraceAuditNo, 
        BigDecimal amount,
        java.util.Date transactionDateTime,
        String dataCaptureIndicator, 
        String track2Data,      
        int transactionCode, 
        int carTerminalType, 
        int carRecordType,
        String approvalCodeIso,
        int submissionCurrency,
        int statusSuccess,
        int statusFailure) throws SQLException {
        this(
            conn,
            terminalID,
            invoiceNo, 
            sysTraceAuditNo, 
            amount,
            transactionDateTime,
            dataCaptureIndicator, 
            track2Data,         
            transactionCode,
            carTerminalType,
            carRecordType,
            approvalCodeIso,
            submissionCurrency,
            false,
            null,
            statusSuccess,
            statusFailure
        );
    }

    /**
     * Constructs TransactionRegister.
     * This is the most general form of the constructor
     * 
     * @param conn DB connection
     * @param terminalID terminal ID
     * @param invoiceNo invoice number
     * @param sysTraceAuditNo sysytem trace audit number
     * @param amount transaction amount
     * @param transactionDateTime transaction time stamp
     * @param dataCaptureIndicator data capure inducator
     * @param track2Data track 2 data
     * @param transactionCode transaction code
     * @param carTerminalType CAR terminal type
     * @param carRecordType CAR record type
     * @param approvalCodeIso approval code
     * @param submissionCurrency submission currency
     * @param isMoto specifies MOTO transaction
     * @param reserveRetrievalNo reservation invoice number (used in ReservationCapture)
     * @param statusSuccess CDS status when the request is registered
     * @param statusFailure return status if the regitration is not successful
     * @throws SQLException when SQL error occurs
     */
    public TransactionRegister(
        Connection conn,
        String terminalID,
        String invoiceNo, 
        String sysTraceAuditNo, 
        BigDecimal amount,
        java.util.Date transactionDateTime,
        String dataCaptureIndicator, 
        String track2Data,      
        int transactionCode, 
        int carTerminalType, 
        int carRecordType,
        String approvalCodeIso,
        int submissionCurrency,
        boolean isMoto,
        String reserveRetrievalNo,
        int statusSuccess,
        int statusFailure) throws SQLException {
            
        trmID = terminalID;
        boolean releaseConn = false;
        CallableStatement cstmt = null;
        try {
            
            if (conn == null) {
                conn = Connector.getConnection();
                releaseConn = true;
            }
            
            cstmt = conn.prepareCall("{call REGISTER_REQUEST (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)}");
            /*          Fixed values in REGISTER_REQUEST
                         rectype number = 11;
                         formatVersion number = 1
                         foreignCurrAmount number = 0
                         formatSalesDesc number = 0
                         lSalesDescRec number = 0
            */
            // Input Parameters
            cstmt.setString(1, terminalID); //terminalid IN varchar2, CDS.CD_0010
            cstmt.setString(2, invoiceNo); //invoiceno IN varchar2, CDS.CD_0015
            cstmt.setString(3, sysTraceAuditNo); //systrace IN varchar2, CDS.CD_0020
            cstmt.setBigDecimal(4, amount); //amount IN number, CDS.CD_1110
            cstmt.setInt(5, Integer.parseInt(new SimpleDateFormat("yyyyMMdd").format(transactionDateTime))); //tcdate IN number, CDS.CD_1150
            cstmt.setInt(6, Integer.parseInt(new SimpleDateFormat("HHmm").format(transactionDateTime))); //tctime IN number, CDS.CD_1160
            cstmt.setString(7, dataCaptureIndicator); //dataCaptureIndicator IN varchar2, CDS.CD_1170
            cstmt.setString(8, track2Data); //track2data IN varchar2, CDS.CD_1180
            cstmt.setInt(9, transactionCode); //transaction_code IN number, CDS.CD_1090
            cstmt.setInt(10, isMoto ? 1 : 0); // is_moto IN int
            cstmt.setInt(11, carTerminalType); //trm_type IN number, CDS.CD_1280
            cstmt.setInt(12, carRecordType); //record_type IN number, CDS.CD_1290
            cstmt.setString(13, approvalCodeIso); //Approval code in case of Refferal. CDS.CD_1190
            cstmt.setInt(14, submissionCurrency); //currency_code IN number
            cstmt.setString(15, reserveRetrievalNo); //reserve_retrieval_no IN varchar2, CDS.CD_1200
            cstmt.setInt(16, statusSuccess);
            cstmt.setInt(17, statusFailure);
            

            // Output parameters
            cstmt.registerOutParameter(18, Types.NUMERIC); //status OUT number, CDS.CD_0030
            cstmt.registerOutParameter(19, Types.NUMERIC); //csdid OUT number, CDS.CD_0000
            cstmt.registerOutParameter(20, Types.NUMERIC); //acq_id OUT number, CTR.AQ_0000 ->  CDS.CD_1000
            cstmt.registerOutParameter(21, Types.NUMERIC); //acq_manufacturer_id OUT number, ACQ.AQ_1250 -> CDS.CD_1001
            cstmt.registerOutParameter(22, Types.VARCHAR); //trm_acq_id OUT varchar2, SUBSTR(CND.CN_0000,1,6) ->  CDS.CD_1020
            cstmt.registerOutParameter(23, Types.VARCHAR); //acq_host OUT varchar2, ACQ.AQ_1330
            cstmt.registerOutParameter(24, Types.VARCHAR); //acq_port OUT varchar2, ACQ.AQ_1340
            cstmt.registerOutParameter(25, Types.VARCHAR); //trm_options OUT varchar2, TRM.TM_1250 || TRM.TM_1240
            cstmt.registerOutParameter(26, Types.NUMERIC); //seqno OUT number, CDS.CD_1060
            cstmt.registerOutParameter(27, Types.VARCHAR); //ctr_vu_number OUT varchar2, CTR.CTR_1050 -> CDS.CD_0000
            cstmt.registerOutParameter(28, Types.NUMERIC); //card_type_id
            cstmt.registerOutParameter(29, Types.VARCHAR); //retrievalRefNo



            // Now!
            cstmt.execute();
            this.status = cstmt.getInt(18);
            this.cdsID = cstmt.getBigDecimal(19);
            this.acqNII = cstmt.getInt(20);
            this.acqManufacturerID = cstmt.getInt(21);
            this.trmACQID = cstmt.getString(22);
            this.carSeqNo = cstmt.getInt(26);
            this.vUNumber = cstmt.getString(27);
            this.cardType = cstmt.getInt(28);
            this.retrievalRefNo = cstmt.getString(29);
        } finally {
            if (cstmt != null) {
                try {
                    cstmt.close();
                } catch (SQLException e) {
                    logger.warn(e);
                }
            }
            if (releaseConn && conn != null) {
                Connector.releaseConnection(conn);
            }
        }
    }
    
    /**
     * Returns card type.
     * @return cardType
     */
    public int getCardType() {
        return cardType;
    }

    /**
     * Returns terminal acquirer ID.
     * @return TrmAcqID
     */
    public String getTrmAcqID() {
        return trmACQID;
    }

    /**
     * Returns VU number.
     * @return VUNumber
     */
    public String getVUNumber() {
        return vUNumber;
    }

    /**
     * Returns acuirer manufacturer ID.
     * @return acqManufacturerID
     */
    public int getAcqManufacturerID() {
        return acqManufacturerID;
    }

    /**
     * Returns acquirer NII.
     * @return acqNII
     */
    public int getAcqNII() {
        return acqNII;
    }
    
    /**
     * Returns the cdsID.
     * @return BigDecimal
     */
    public BigDecimal getCdsID() {
        return cdsID;
    }

    /**
     * Returns the status.
     * @return status
     */
    public int getStatus() {
        return status;
    }

    /**
     * Returns the CAR sequence number.
     * @return carSeqNo
     */
    public int getCarSeqNo() {
        return carSeqNo;
    }

    /**
     * Returns the terminal ID.
     * @return trmID
     */
    public String getTrmID() {
        return trmID;
    }

    /**
     * Returns the Retrieval Reference Number.
     * @return retrievalRefNo
     */
    public String getRetrivalRefNo() {
        return retrievalRefNo;
    }

    /**
     * Returns a string representation of output parameters.
     * @return string representation of output parameters
     */
    public String toString() {
        StringBuffer buf = new StringBuffer();

        buf.append("\nTransaction Context from TransactionRegister");
        buf.append("]\n\tAcqNII: [");
        buf.append(getAcqNII());
        buf.append("]\n\tAcqManufacturer: [");
        buf.append(getAcqManufacturerID());
        buf.append("]\n\tCardType: [");
        buf.append(getCardType());
        buf.append("]\n\tTrmAcqID: [");
        buf.append(getTrmAcqID());
        buf.append("]\n\tVUNumber: [");
        buf.append(getVUNumber());
        buf.append("]\n\tCardType: [");
        buf.append(getCardType());
        buf.append("]\nEnd of Context\n");

        return buf.toString();
    }
    
}
