package com.commtrain.PMS.iso;

import org.jpos.iso.ISOHeader;
import org.jpos.iso.ISOUtil;

/**
 * TPDU Header
 * 
 */
public class IENHeader implements ISOHeader {

    /**
     * the ID used in Hypercom terminals (0x60)
     */
    public static final byte HYPERCOM_ID = (byte) 0x60;

    /**
     * the length of the header (5)
     */
    private static final int LENGTH = 5;

    /**
     * the header
     */
    private byte[] header;

    /**
     * Default constructor
     */
    public IENHeader() {
        this("0000", "0000");
    }

    /**
     * Constructs TPDU header with specified source and destination
     * @param source source
     * @param destination destionation
     */
    public IENHeader(String source, String destination) {
        super();
        header = new byte[LENGTH];
        setId(HYPERCOM_ID);
        setSource(source);
        setDestination(destination);
    }

    /**
     * Constructs TPDU header with specified destination
     * @param destination destionation
     */
    public IENHeader(String destination) {
        // The IENChannel swaps source and destination
        this(destination, "0000");
    }
    
    /**
     * Create a new header from a byte array.
     * @param header the byte array
     * @return The Number of bytes consumed.
     * @see org.jpos.iso.ISOHeader#unpack(byte[])
     */
    public int unpack(byte[] header) {
        this.header = header;
        return header != null ? header.length : 0;
    }

    /**
     * Sets ID byte.
     * @param id ID byte
     */
    public void setId(byte id) {
        header[0] = id;
    }

    /**
     * Set the Destination address in this IENHeader.
     * @param dest the destination address
     * @see org.jpos.iso.ISOHeader#setDestination(String)
     */
    public void setDestination(String dest) {
        byte[] h = ISOUtil.hex2byte(dest);
        System.arraycopy(h, 0, header, 1, 2);
    }

    /**
     * Set the Source address in this IENHeader.
     * @param src the source address
     * @see org.jpos.iso.ISOHeader#setSource(String)
     */
    public void setSource(String src) {
        byte[] h = ISOUtil.hex2byte(src);
        System.arraycopy(h, 0, header, 3, 2);
    }
    
    /**
     * Gets ID byte.
     * @return ID byte
     */
    public byte getId() {
        return header[0];
    }

    /**
     * Return the source address in this ISOHeader.
     * @return the source address
     * @see org.jpos.iso.ISOHeader#getSource()
     */
    public String getSource() {
        byte[] result = new byte[2];
        System.arraycopy(header, 3, result, 0, 2);
        return ISOUtil.dumpString(result);
    }


    /**
     * Return the destination address in this ISOHeader.
     * @return the destination address
     * @see org.jpos.iso.ISOHeader#getDestination()
     */
    public String getDestination() {
        byte[] result = new byte[2];
        System.arraycopy(header, 1, result, 0, 2);
        return ISOUtil.dumpString(result);
    }

    /**
     * Swap the source and destination addresses in this IENHeader
     * (if they exist).
     * @see org.jpos.iso.ISOHeader#swapDirection()
     */
    public void swapDirection() {
        if (header != null && header.length >= LENGTH) {
            byte[] source = new byte[2];
            System.arraycopy(header, 3, source, 0, 2);
            System.arraycopy(header, 1, header, 3, 2);
            System.arraycopy(source, 0, header, 1, 2);
        }
    }
    /** 
     * Clone this Header.
     * @see java.lang.Object#clone()
     */
    public Object clone() {
        try {
            IENHeader h = (IENHeader) super.clone();
            if (this.header != null) {
                h.header = (byte[]) this.header.clone();
            }
            return h;
        } catch (CloneNotSupportedException e) {
            throw new InternalError();
        }
    }

    /**
     * Return this header as byte array.
     * @see org.jpos.iso.ISOHeader#pack()
     */
    public byte[] pack() {
        return header;
    }


    /**
     * Returns the number of bytes in this IENHeader
     * @return the number of bytes in this IENHeader
     * @see org.jpos.iso.ISOHeader#getLength()
     */
    public int getLength() {
        return header != null ? header.length : 0;
    }
    
};
