package com.commtrain.PMS.transaction;

import java.io.IOException;
import java.io.InterruptedIOException;
import java.math.BigDecimal;
import java.sql.SQLException;

import org.apache.log4j.Priority;
import org.jpos.iso.BaseChannel;
import org.jpos.iso.ClientChannel;
import org.jpos.iso.ISOException;
import org.jpos.iso.ISOMsg;
import org.jpos.iso.ISOUtil;
import org.jpos.iso.channel.NACChannel;

import com.commtrain.PMS.Props;
import com.commtrain.PMS.db.ProcsFuncs;
import com.commtrain.PMS.db.TransactionContext;
import com.commtrain.PMS.iso.CARRequest;
import com.commtrain.PMS.iso.CARResponse;
import com.commtrain.PMS.iso.IENHeader;
import com.commtrain.PMS.iso.PayGatewayChannel;
import com.commtrain.PMS.iso.packager.CARPackager;
import com.commtrain.PMS.logging.Logger4J;

/**
 *
 */
abstract public class OnlineTransaction extends Transaction {
    private static final int DEFAULT_AUTHORIZATION_TIMEOUT = 10000; //miliseconds
    private static final int DEFAULT_IEN_PORT = 1000;
    private static org.jpos.util.Logger acqLogger = new Logger4J(Priority.INFO_INT);
    private static org.apache.log4j.Logger logger = org.apache.log4j.Logger.getLogger(OnlineTransaction.class);

    protected CARRequest acqRequest;
    protected CARResponse acqResponse;
    protected int acqNII = 0;
    protected ClientChannel channel;

    protected OnlineTransaction(ISOMsg isoRequest){
        super(isoRequest);
        cdsID = new BigDecimal("-1");
    }

    protected void sendAcqRequest(ISOMsg reqMsg) throws SQLException, ISOException, IOException {
        int nii = (acqNII == 0) ? transactionContext.getAcqNII() : acqNII;
        String niiHexStr = ISOUtil.padleft(Integer.toString(nii, 10), 4, '0');

        reqMsg.setHeader(new IENHeader(niiHexStr));
        channel.connect();
        channel.send(reqMsg);
    }

    protected CARResponse recieveAcqResponse() throws ISOException, IOException {
        try {
          	return new CARResponse(channel.receive());
        } finally {
            channel.disconnect();
        }
    }

    /**
     * @see com.commtrain.PMS.transaction.Transaction#process()
     * 
     */
    public ISOMsg process() {
        try {
	    	calcTransactionContext();
            convertRequest();

            if (status != Transaction.STATUS_RQ_CONVERTED + getTransactionKind())
                return createErrorMessage(isoRequest, cdsID.toString(), null, RESPONSE_CODE_INVALID_TRANSACTION);

	    	setupAcquierCommunication();
			sendAcqRequest(acqRequest);
            acqResponse = recieveAcqResponse();

            convertResponse();
            return isoResponse;
        } catch (Exception e) {
            
            if(e instanceof InterruptedIOException && transactionContext!=null){
                switch (transactionContext.getAcqCommMethod()) {
                    case TransactionContext.COMM_METHOD_IP :
                        logger.error("java.io.InterruptedIOException: Read timed out with "+transactionContext.getAcqIPHost()+":"+transactionContext.getAcqIPPort());
                        break;
                
                    default :
                        int nii = (acqNII == 0) ? transactionContext.getAcqNII() : acqNII;
                        logger.error("java.io.InterruptedIOException: Read timed out with NII="+nii);
                        break;
                }
            } else{
                logger.error("Stack Trace", e);
            }
            
            if (status == STATUS_RQ_RECEIVED) {
                ProcsFuncs.addFakeCDSRecord(isoRequest);
            }

            String retrievalReferenceNum = TransactionUtils.getString(isoResponse, RETRIEVAL_REFERENCE_NUM, "");
            String approvalCode = acqResponse != null && acqResponse.hasField(AUTHORIZATION_IDENTIFICATION_RESPONSE) ? acqResponse.getString(AUTHORIZATION_IDENTIFICATION_RESPONSE) : null;
            String responseCode = TransactionUtils.getString(isoResponse, RESPONSE_CODE, RESPONSE_CODE_REENTER_TRANSACTION);
            return createErrorMessage(isoRequest, retrievalReferenceNum, approvalCode ,responseCode);
        }
    }

    /**
     * Sets autorizationChannel and autorizationTimeout
     */
    public void setAuthorizationChannel(ClientChannel channel) {
    	this.channel = channel;
    }

    /**
     * Converts ISORequest -> ACQRequest.
     */
    public abstract void convertRequest() throws ISOException, SQLException;

    /**
     * Converts ACQResponse -> ISOResponse.
     */
    public abstract void convertResponse() throws ISOException, SQLException;

    /**
     * Sets MANIFACTURE_ID, TERMINAL_ID, SEQUENCE_NO_ID, VU_NUMBER in a way that is common for CAR Requests.
     */
    protected static CARRequest fillCommonCARFields(CARRequest req, TransactionContext ctx, int carSeqNo) throws ISOException {
        req.setField(CARRequest.MANIFACTURE_ID, ctx.getAcqManufacturerID().toString());
        req.setField(CARRequest.TERMINAL_ID, ctx.getTrmAcqID());
        req.setField(CARRequest.SEQUENCE_NO_ID, ISOUtil.padleft(Integer.toString(carSeqNo), 4, '0'));
        req.setField(CARRequest.VU_NUMBER, ctx.getVUNumber());
        return req;
    }

    public static String convertCar2IsoResponseCode(String carResponseCode) {

        int carResNo = 0;
        String isoResponseCode;
        try {
            carResNo = Integer.parseInt(carResponseCode);
        } catch (NumberFormatException e) {
            carResNo = 79;
        }
        switch (carResNo) {
            case 1 :
                isoResponseCode = "00"; // APPROVED
                break;

            case 59 :
            case 60 :
                isoResponseCode = "02"; // REFERRAL
                break;

            case 12 :
                isoResponseCode = "12";// INVALID TRANSACTION
                break;

            case 13 :
                isoResponseCode = "13";// INVALID AMOUNT
                break;

            case 11 :
                isoResponseCode = "30";// FORMAT ERROR
                break;

            case 54 :
                isoResponseCode = "54";// EXPIRED CARD
                break;

            case 41 :
            case 42 :
            case 43 :
                isoResponseCode = "43";// STOLEN CARD, PICK UP
                break;

            default :
            case 50 :
            case 51 :
            case 79 :
                isoResponseCode = "79";// DECLINED - CVV2
                break;
        }
        try {
            isoResponseCode = ISOUtil.padleft(isoResponseCode, 2, '0');
        } catch (ISOException e) {
            isoResponseCode = "79";
        }
        return isoResponseCode;
    }
    
	protected void setupAcquierCommunication(){
        String channelRealm;
        BaseChannel authorizationChannel;
        CARPackager carPackager = new CARPackager();
		logger.debug("Communication Method: "+transactionContext.getAcqCommMethod());
		switch (transactionContext.getAcqCommMethod()) {
            case TransactionContext.COMM_METHOD_IP :
            	authorizationChannel = new PayGatewayChannel(transactionContext.getAcqIPHost(), transactionContext.getAcqIPPort(), carPackager);
		        channelRealm = "PayGateway";
                break;
                
            default :
                String hostName = Props.get("IENHost", "not defined authorization host");
                int port = Props.getInt("IENPort", DEFAULT_IEN_PORT);
                authorizationChannel = new NACChannel(hostName, port, carPackager, new byte[5]);
                channelRealm = "IEN";
                break;
        }
        
        carPackager.setLogger(acqLogger, channelRealm + ".CARPackager");
        authorizationChannel.setLogger(acqLogger, channelRealm + ".channel");
        try {
            int timeout = Props.getInt("IENTimeout", DEFAULT_AUTHORIZATION_TIMEOUT);
            authorizationChannel.setTimeout(timeout);
        } catch (Exception e) {
            logger.warn("Warning! Unable to set timeout!", e);
        }
        setAuthorizationChannel(authorizationChannel);
   }
    
    public int getTransactionKind() {
        //Normal Credit Transaction
        return STATUS_DETAIL_NORMAL_CREDIT;
    }

}
