package com.commtrain.PMS.transaction;

import java.text.SimpleDateFormat;

import org.jpos.iso.ISOException;
import org.jpos.iso.ISOMsg;

import com.commtrain.PMS.iso.AdditionalData;
import com.commtrain.PMS.iso.CVV2Data;

//import com.commtrain.PMS.iso.packager.MANORPackager;

/**
 * 
 */
public class TransactionUtils {

    public static final String POS_ENTRY_MODE_AUTO = "02";
    public static final String POS_ENTRY_MODE_MANUAL = "01";

    public static final String CDS_ENTRY_MODE_AUTO = "A";
    public static final String CDS_ENTRY_MODE_MANUAL = "M";

    public static final int CARD_DETAILS_DEPEND_ON_FLAG = 1;
    public static final int CARD_DETAILS_WHICH_PRESENTS = 2;
    public static final int CARD_DETAILS_MAG_STRIPE = 3;
    public static final int CARD_DETAILS_MANUAL = 4;
    
    private static SimpleDateFormat __timeFormatter;
    private static SimpleDateFormat __dateFormatter;

    public static String getCVV2(ISOMsg isoRequest) throws ISOException {
        String tek = "";
        if (isoRequest.hasField(Transaction.ADDITIONAL_DATA)) {
            AdditionalData additionalData = (AdditionalData) isoRequest.getComponent(Transaction.ADDITIONAL_DATA);
            if (additionalData.hasField(CVV2Data.FIELD_NUMBER)) {
                CVV2Data cvv2Data = (CVV2Data) additionalData.getComponent(CVV2Data.FIELD_NUMBER);
                tek = cvv2Data.getCVV2Value().trim();
            }
        }
        return tek;
    }

    public static void setDateTimeInISOMsg(ISOMsg isoMsg, java.util.Date date) throws ISOException {
    	if(__timeFormatter==null || __dateFormatter==null){
    		__timeFormatter = new SimpleDateFormat("HHmmss");
    		__dateFormatter = new SimpleDateFormat("MMdd");
    	}
    	
        isoMsg.set(Transaction.TIME_LOCAL_TRANSACTION, __timeFormatter.format(date));
        isoMsg.set(Transaction.DATE_LOCAL_TRANSACTION, __dateFormatter.format(date));
    }

    public static String getDataCaptureIndicator(ISOMsg isoRequest) throws ISOException {
        String posEntryMode = isoRequest.getString(Transaction.POS_ENTRY_MODE);
        if (posEntryMode == null)
            throw new ISOException("POS_ENTRY_MODE missing in ISO Request");
        return (isoRequest.getString(Transaction.POS_ENTRY_MODE).startsWith(POS_ENTRY_MODE_AUTO) ? CDS_ENTRY_MODE_AUTO : CDS_ENTRY_MODE_MANUAL);
    }

    public static String getCardDetails(ISOMsg isoRequest) throws ISOException {
        return getCardDetails(isoRequest, CARD_DETAILS_DEPEND_ON_FLAG);
    }

    public static String getCardDetails(ISOMsg isoRequest, int mode) throws ISOException {
        boolean dataFromTrack2;

        switch (mode) {
            case CARD_DETAILS_MAG_STRIPE :
                dataFromTrack2 = true;
                break;

            case CARD_DETAILS_MANUAL :
                String strError = "";

                if (!isoRequest.hasField(Transaction.PAN_PRIMARY_ACC_NUM))
                    strError += ", " + "PAN missing in ISO Request";

                if (!isoRequest.hasField(Transaction.DATE_EXPIRATION))
                    strError += ", " + "DateExp missing in ISO Request";

                if (strError.length() > 0)
                    throw new ISOException(strError.substring(2));
                dataFromTrack2 = false;
                break;

            case CARD_DETAILS_DEPEND_ON_FLAG :
                String posEntryMode = isoRequest.getString(Transaction.POS_ENTRY_MODE);
                if (posEntryMode == null)
                    throw new ISOException("POS_ENTRY_MODE missiong is ISO Request");
                if (posEntryMode.startsWith(TransactionUtils.POS_ENTRY_MODE_AUTO)) {
                    dataFromTrack2 = true;
                } else if (posEntryMode.startsWith(TransactionUtils.POS_ENTRY_MODE_MANUAL)) {
                    dataFromTrack2 = false;
                } else
                    throw new ISOException("Unknown POS_ENTRY_MODE in ISO Request");
                break;

            case CARD_DETAILS_WHICH_PRESENTS :
                boolean hasPAN = isoRequest.hasField(Transaction.PAN_PRIMARY_ACC_NUM);
                boolean hasTrack2Data = isoRequest.hasField(Transaction.TRACK_2_DATA);

                if (hasPAN && hasTrack2Data)
                    throw new ISOException("Both PAN and TRACK_2_DATA are included.");
                if (hasTrack2Data) {
                    dataFromTrack2 = true;
                } else
                    dataFromTrack2 = false;
                break;
            default :
                throw new RuntimeException("Unknown CARD_DETAILS_MODE");
        }

        String cardDetails = null;

        if (dataFromTrack2) {
            cardDetails = ";" + isoRequest.getString(Transaction.TRACK_2_DATA);
            if (!cardDetails.endsWith("?"))
                cardDetails += "?";
        } else {
            cardDetails = ";" + isoRequest.getString(Transaction.PAN_PRIMARY_ACC_NUM) + "=" + isoRequest.getString(Transaction.DATE_EXPIRATION) + TransactionUtils.getCVV2(isoRequest);
        }
        return cardDetails;
    }

    /**
     * Uses the isoRequest to distinguish the Transaction code (CD_1090).
     * @return int
     */
    public static int getTransactionCode(ISOMsg isoRequest) {
        String procCode = isoRequest.getString(Transaction.PROCESSING_CODE);
        String mti = isoRequest.getString(Transaction.MESSAGE_TYPE_INDICATOR);
        if (procCode.startsWith("0000"))
            //	Authorization, Sale, Off-line Sale, Sales Completion
            return 5;
        else if (procCode.startsWith("0200")) {
            //	Re-Authorization, Void Sale on-line, Adjust Sale
            return 5;
        } else if (procCode.startsWith("2000"))
            //	Refund, Off-line Refund, 
            return 6;
        else if (procCode.startsWith("2200")) {
            //	Payment Reversal, Void Refund on-line, Adjust Refund
            return 6;
        } else if (procCode.startsWith("3000") && "0100".equals(mti))
            //	Pre-Authorization. In WebShop used as Resrevation
            return 5;
        else if (procCode.startsWith("0220") && "0220".equals(mti))
            //	WebShop Refund
            return 6;
        else if (procCode.startsWith("3000") && "0200".equals(mti))
            //	Available Funds. Used as Direct Sale in WebShop
            return 5;
        else if (procCode.startsWith("4000") && ("0200".equals(mti) || "0400".equals(mti)))
            //  Reservation
            return 5;
        else if (procCode.startsWith("4100") && ("0200".equals(mti) || "0400".equals(mti)))
            //  Reservation Capture
            return 5;
        else
            return 0;
    }
    

    /**
     * Bizarre (non logic) recommendation from certifiers to change code 5->6 and vice versa (ex. 101 records in CDS)
     * All others codes remain the same (ex 0 ->0)
     * Uses the isoRequest to distinguish the Transaction code (CD_1090).
     * @return int
     */
    public static int getTransactionCodeInverse(ISOMsg isoRequest) {
    	int normalCode = getTransactionCode(isoRequest);
    	if(normalCode==5)
    		return 6;
    	
    	if(normalCode==6)
    		return 5;

   		return normalCode;
    }

    /**
     * Uses the isoRequest to distinguish the CARTerminal type.
     * @return int
     * Currently supported types:
     *  Normal
     *  Internet w/o security
     *  Internet SSL
     */
    public static int getCARTerminalType(ISOMsg isoRequest) {
        String posCondCode = isoRequest.getString(Transaction.POS_CONDITION_CODE);
        if (Transaction.POS_CONDITION_CODE_SECURE.equals(posCondCode))
            return Transaction.CAR_TERMINAL_TYPE_INTERNET_SECURE;
        else if (Transaction.POS_CONDITION_CODE_UNSECURE.equals(posCondCode))
            return Transaction.CAR_TERMINAL_TYPE_INTERNET_UNSECURE;
        else
            return Transaction.CAR_TERMINAL_TYPE_NORMAL;

    }

    public static String getString(ISOMsg m, int fldno, String defaultValue) {
        return m.hasField(fldno) ? m.getString(fldno) : defaultValue;
    }

    public static String getString(ISOMsg m, int fldno) {
        return getString(m, fldno, null);
    }

    public static String getCurrencyCode(ISOMsg m) {
        return getString(m, Transaction.CURRENCY_CODE_TRANSACTION, Transaction.POS_LOCAL_CURRENCY);
    }

    public static int getIntCurrencyCode(ISOMsg m) {
        return Integer.parseInt(getCurrencyCode(m));
    }

}
