package com.commtrain.util;

import java.text.MessageFormat;
import java.util.Locale;
import java.util.MissingResourceException;
import java.util.ResourceBundle;

/**
 * Wrapper class for locale-specific strings.
 * The strings are kept in resource bundle.
 * The name of the bundle is fixed.
 *
 * @see java.util.ResourceBundle; java.text.MessageFormat;
 *
 * 
 */

public class MessageCatalog {

    /**
     * The name of the bundle used by this class.
     */
    private String bundleName = null;

    /**
     * Constructs message catalog. Bundle's name is derived from class name.
     * 
     * @param packageMember Bundle's package is packageMember's package
     *             plus folder "resources".
     *             The unqualified name of the bundle is set to "Messages".
     */
    public MessageCatalog(Class packageMember) {
        super();
        bundleName =
            MessageCatalog.bundleName(packageMember, "Messages", "resources");
    }

    /**
     * Constructs fully qualified bundle name.
     * 
     * @param packageMember Bundle's package is derived
     *             from the packageMember's package.
     * @param bundle The unqualified (short) name of the bundle.               
     * @param folder This folder is added to bundle's package.
     * @return Fully qualified bundle name.
     */
    public static String bundleName(
        Class packageMember,
        String bundle,
        String folder) {

        String result = packageMember.getName();
        int index = result.lastIndexOf('.');
        return (index != -1 ? result.substring(0, index) + "." : "")
            + folder
            + '.'
            + bundle;
    }

    /**
     * Constructs locale from string description.
     * 
     * @param description The string representation of the locale
     * @return Constructed locale.
     */
    public static Locale createLocale(String description) {
        int len = description.length();
        Locale result;

        if (len == 2) {
            result = new Locale(description, "");
        } else if (len >= 5) {
            result =
                new Locale(
                    description.substring(0, 2),
                    description.substring(3, 5));
        } else {
            result = Locale.getDefault();
        }
        return result;
    }


    /**
     * Gets string from the resource bundle by key and locale.
     * 
     * @param key The given key
     * @param locale The specific locale
     * @return The locale-specific resource.
     */
    public String getString(String key, Locale locale) {
        try {
            return ResourceBundle.getBundle(bundleName, locale).getString(key);
        } catch (MissingResourceException e) {
            return key;
        }
    }

    /**
     * Gets string from the resource bundle by key and locale description.
     * 
     * @param key The given key
     * @param localeDescription The specific locale
     * @return The locale-specific resource.
     */
    public String getString(String key, String localeDescription) {
        return getString(key, createLocale(localeDescription));
    }

    /**
     * Gets formated string from the resource bundle by key, 
     * locale and format parameters.
     * 
     * @param key The given key. The correspondig string in the resource bundle
     *             must be message format.
     * @param locale The specific locale
     * @param parameters The format parameters
     * @return The locale-specific resource.
     *
     * @see java.text.MessageFormat#format()
     */
    public String getString(String key, Locale locale, Object[] parameters) {
        MessageFormat messageFormat = new MessageFormat("");
        messageFormat.setLocale(locale);
        messageFormat.applyPattern(getString(key, locale));
        return messageFormat.format(parameters);
    }

    /**
     * Gets formated string from the resource bundle by key, 
     * locale description and format parameters.
     * 
     * @param key The given key. The correspondig string in the resource bundle
     *             must be message format.
     * @param localeDescription The specific locale
     * @param parameters The format parameters
     * @return The locale-specific resource.
     *
     * @see java.text.MessageFormat#format()
     */
    public String getString(
        String key,
        String localeDescription,
        Object[] parameters) {
        return getString(key, createLocale(localeDescription), parameters);
    }

}
